@@ -16,7 +16,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA  02111-1307, USA.
  *
- * $Id: compiler.h,v 1.10 2004/03/03 21:35:38 curt Exp $
+ * $Id: compiler.h,v 1.11 2004/04/30 00:44:04 andy Exp $
  *
  **************************************************************************/
 
@@ -149,13 +149,6 @@
 
 #endif // __GNUC__
 
-#if defined( __MINGW32__ )
-#  define bcopy(from, to, n) memcpy(to, from, n)
-#  define FG_MEM_COPY(to,from,n) memcpy(to, from, n)
-#  define isnan _isnan
-#  define snprintf _snprintf
-#endif
-
 /* KAI C++ */
 #if defined(__KCC)
 
