@@ -23,7 +23,9 @@ typedef enum {
     SG_IO        = 0x00001000,
     SG_CLIPPER   = 0x00002000,
     SG_NETWORK   = 0x00004000,
-    SG_UNDEFD    = 0x00008000, // For range checking
+    SG_ATC       = 0x00008000,
+    SG_NASAL     = 0x00010000,
+    SG_UNDEFD    = 0x00020000, // For range checking
 
     SG_ALL     = 0xFFFFFFFF
 } sgDebugClass;
@@ -33,11 +35,12 @@ typedef enum {
  * Define the possible logging priorities (and their order).
  */
 typedef enum {
-    SG_BULK,	    // For frequent messages
-    SG_DEBUG, 	    // Less frequent debug type messages
-    SG_INFO,        // Informatory messages
-    SG_WARN,	    // Possible impending problem
-    SG_ALERT       // Very possible impending problem
+    SG_BULK = 1,       // For frequent messages
+    SG_DEBUG,          // Less frequent debug type messages
+    SG_INFO,           // Informatory messages
+    SG_WARN,           // Possible impending problem
+    SG_ALERT           // Very possible impending problem
     // SG_EXIT,        // Problem (no core)
     // SG_ABORT        // Abandon ship (core)
 } sgDebugPriority;
+
