@@ -19,7 +19,7 @@
 // Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 // Boston, MA  02111-1307, USA.
 //
-// $Id: stardata.cxx,v 1.6 2001/12/28 23:37:34 curt Exp $
+// $Id: stardata.cxx,v 1.2 2002/12/31 14:47:35 david Exp $
 
 
 #include <simgear/debug/logstream.hxx>
@@ -27,7 +27,7 @@
 
 #include "stardata.hxx"
 
-#if defined (_MSC_VER) || defined (SG_HAVE_NATIVE_SGI_COMPILERS)
+#if defined (_MSC_VER)
   SG_USING_STD(getline);
 #endif
 
