@@ -3,16 +3,14 @@
 #include STL_IOSTREAM
 #include "lowlevel.hxx"
 
-#if !defined(SG_HAVE_NATIVE_SGI_COMPILERS)
-SG_USING_STD(cout);
-SG_USING_STD(endl);
-#endif
-
 
 static const int sgEndianTest = 1;
 #define sgIsLittleEndian (*((char *) &sgEndianTest ) != 0)
 #define sgIsBigEndian    (*((char *) &sgEndianTest ) == 0)
 
+SG_USING_STD(cout);
+SG_USING_STD(endl);
+
 
 int main() {
     cout << "This machine is ";
@@ -23,6 +21,8 @@ int main() {
     }
     cout << "endian" << endl;
 
+    cout << "sizeof(short) = " << sizeof(short) << endl;
+
     short s = 1111;
     cout << "short s = " << s << endl;
     sgEndianSwap((unsigned short *)&s);
