@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: sg_binobj.cxx,v 1.4 2003/03/02 14:52:05 david Exp $
+// $Id: sg_binobj.cxx,v 1.5 2004/07/14 22:01:15 curt Exp $
 //
 
 
@@ -116,7 +116,7 @@ public:
 
 // calculate the center of a list of points, by taking the halfway
 // point between the min and max points.
-static Point3D calc_center( point_list& wgs84_nodes ) {
+Point3D sgCalcCenter( point_list& wgs84_nodes ) {
     Point3D p, min, max;
 
     if ( wgs84_nodes.size() ) {
@@ -1145,7 +1145,7 @@ bool SGBinObject::write_ascii( const string& base, const string& name,
 	    for ( i = start; i < end; ++i ) {
 		for ( j = 0; j < (int)tris_v[i].size(); ++j ) {
 		    group_nodes.push_back( wgs84_nodes[ tris_v[i][j] ] );
-		    bs_center = calc_center( group_nodes );
+		    bs_center = sgCalcCenter( group_nodes );
 		    bs_radius = sgCalcBoundingRadius( bs_center, group_nodes );
 		}
 	    }
@@ -1196,7 +1196,7 @@ bool SGBinObject::write_ascii( const string& base, const string& name,
 	    for ( i = start; i < end; ++i ) {
 		for ( j = 0; j < (int)strips_v[i].size(); ++j ) {
 		    group_nodes.push_back( wgs84_nodes[ strips_v[i][j] ] );
-		    bs_center = calc_center( group_nodes );
+		    bs_center = sgCalcCenter( group_nodes );
 		    bs_radius = sgCalcBoundingRadius( bs_center, group_nodes );
 		}
 	    }
