@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: sg_socket_udp.cxx,v 1.1 2001/11/12 04:47:59 curt Exp $
+// $Id: sg_socket_udp.cxx,v 1.2 2004/02/07 21:36:20 david Exp $
 
 
 #include <simgear/compiler.h>
@@ -162,7 +162,7 @@ int SGSocketUDP::write( const char *buf, const int length ) {
     bool error_condition = false;
 
     if ( sock.send( buf, length, 0 ) < 0 ) {
-	SG_LOG( SG_IO, SG_ALERT, "Error writing to socket: " << port );
+	SG_LOG( SG_IO, SG_WARN, "Error writing to socket: " << port );
 	error_condition = true;
 	return 0;
     }
