@@ -20,7 +20,7 @@
 // Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 // Boston, MA  02111-1307, USA.
 //
-// $Id: sg_path.cxx,v 1.8 2004/03/21 21:54:51 ehofman Exp $
+// $Id: sg_path.cxx,v 1.11 2004/07/05 16:39:02 ehofman Exp $
 
 
 #include <simgear/compiler.h>
@@ -43,7 +43,7 @@ static const char sgDirPathSep = '/';
 static const char sgDirPathSepBad = '\\';
 #endif
 
-#if defined( WIN32 )
+#if defined( WIN32 ) && !defined(__CYGWIN__)
 static const char sgSearchPathSep = ';';
 #else
 static const char sgSearchPathSep = ':';
@@ -112,6 +112,11 @@ void SGPath::append( const string& p ) {
     fix();
 }
 
+//add a new path component to the existing path string
+void SGPath::add( const string& p ) {
+    append( sgSearchPathSep+p );
+}
+
 
 // concatenate a string to the end of the path without inserting a
 // path separator
@@ -149,7 +154,7 @@ string SGPath::dir() const {
 // get the base part of the path (everything but the extension.)
 string SGPath::base() const {
     int index = path.rfind(".");
-    if (index >= 0) {
+    if ((index >= 0) && (path.find("/", index) == string::npos)) {
 	return path.substr(0, index);
     } else {
 	return "";
@@ -157,9 +162,11 @@ string SGPath::base() const {
 }
 
 // get the extention (everything after the final ".")
+// but make sure no "/" follows the "." character (otherwise it
+// is has to be a directory name containing a ".").
 string SGPath::extension() const {
     int index = path.rfind(".");
-    if (index >= 0) {
+    if ((index >= 0)  && (path.find("/", index) == string::npos)) {
 	return path.substr(index + 1);
     } else {
 	return "";
