@@ -19,7 +19,10 @@
 // Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 // Boston, MA  02111-1307, USA.
 //
-// $Id: sgstream.cxx,v 1.1.1.1 2002/09/07 02:58:19 curt Exp $
+// $Id: sgstream.cxx,v 1.3 2004/07/28 14:13:16 ehofman Exp $
+
+#include <simgear/compiler.h>
+#include STL_STRING 
 
 #include <ctype.h> // isspace()
 
