@@ -20,7 +20,7 @@
 // Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 // Boston, MA  02111-1307, USA.
 //
-// $Id: sky.cxx,v 1.17 2004/04/04 13:41:53 ehofman Exp $
+// $Id: sky.cxx,v 1.18 2004/08/17 08:28:35 ehofman Exp $
 
 
 #include <plib/sg.h>
@@ -266,7 +266,10 @@ void SGSky::modify_vis( float alt, float time_factor ) {
 
 	double ratio = 1.0;
 
-	if ( alt < asl - transition ) {
+        if ( cloud_layers[i]->getCoverage() == SGCloudLayer::SG_CLOUD_CLEAR ) {
+	    // clear layer
+	    ratio = 1.0;
+        } else if ( alt < asl - transition ) {
 	    // below cloud layer
 	    ratio = 1.0;
 	} else if ( alt < asl ) {
