@@ -21,7 +21,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: soundmgr_openal.cxx,v 1.5 2004/07/01 19:05:37 curt Exp $
+// $Id: soundmgr_openal.cxx,v 1.6 2004/08/15 09:24:12 ehofman Exp $
 
 #include <simgear/compiler.h>
 
@@ -61,7 +61,7 @@ inline int (isnan)(double r) { return isnan(r); }
 // constructor
 SGSoundMgr::SGSoundMgr() {
 
-    SG_LOG( SG_GENERAL, SG_ALERT, "Initializing OpenAL sound manager" );
+    SG_LOG( SG_GENERAL, SG_INFO, "Initializing OpenAL sound manager" );
 
     // initialize OpenAL
     alutInit( 0, NULL );
